@@ -13,15 +13,38 @@
 #include "nulstr-util.h"
 #include "parse-util.h"
 #include "path-util.h"
+#include "proc-cmdline.h"
 #include "string-util.h"
 #include "strv.h"
 
+bool fstab_enabled_full(int enabled) {
+        static int cached = -1;
+        bool val = true; /* If nothing specified or the check fails, then defaults to true. */
+        int r;
+
+        /* If 'enabled' is non-negative, then update the cache with it. */
+        if (enabled >= 0)
+                cached = enabled;
+
+        if (cached >= 0)
+                return cached;
+
+        r = proc_cmdline_get_bool("fstab", PROC_CMDLINE_STRIP_RD_PREFIX|PROC_CMDLINE_TRUE_WHEN_MISSING, &val);
+        if (r < 0)
+                log_debug_errno(r, "Failed to parse fstab= kernel command line option, ignoring: %m");
+
+        return (cached = val);
+}
+
 int fstab_has_fstype(const char *fstype) {
         _cleanup_endmntent_ FILE *f = NULL;
         struct mntent *m;
 
         assert(fstype);
 
+        if (!fstab_enabled())
+                return false;
+
         f = setmntent(fstab_path(), "re");
         if (!f)
                 return errno == ENOENT ? false : -errno;
@@ -88,6 +111,9 @@ int fstab_is_mount_point_full(const char *where, const char *path) {
 
         assert(where || path);
 
+        if (!fstab_enabled())
+                return false;
+
         f = setmntent(fstab_path(), "re");
         if (!f)
                 return errno == ENOENT ? false : -errno;
@@ -305,19 +331,25 @@ static char *tag_to_udev_node(const char *tagvalue, const char *by) {
 }
 
 char *fstab_node_to_udev_node(const char *p) {
+        const char *q;
+
         assert(p);
 
-        if (startswith(p, "LABEL="))
-                return tag_to_udev_node(p+6, "label");
+        q = startswith(p, "LABEL=");
+        if (q)
+                return tag_to_udev_node(q, "label");
 
-        if (startswith(p, "UUID="))
-                return tag_to_udev_node(p+5, "uuid");
+        q = startswith(p, "UUID=");
+        if (q)
+                return tag_to_udev_node(q, "uuid");
 
-        if (startswith(p, "PARTUUID="))
-                return tag_to_udev_node(p+9, "partuuid");
+        q = startswith(p, "PARTUUID=");
+        if (q)
+                return tag_to_udev_node(q, "partuuid");
 
-        if (startswith(p, "PARTLABEL="))
-                return tag_to_udev_node(p+10, "partlabel");
+        q = startswith(p, "PARTLABEL=");
+        if (q)
+                return tag_to_udev_node(q, "partlabel");
 
         return strdup(p);
 }
